//+------------------------------------------------------------------
//|                                     Klinger volume oscillator.mq4 
//+------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers    6
#property indicator_color1     Red
#property indicator_color2     Maroon
#property indicator_color3     LimeGreen
#property indicator_color4     DarkGreen
#property indicator_color5     DimGray
#property indicator_color6     DeepSkyBlue     
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     2
#property indicator_width4     2
#property indicator_width5     2
#property indicator_width6     2
#property indicator_level1     0
#property indicator_levelcolor MediumOrchid

//
//
//
//
//

extern string TimeFrame                 = "Current time frame";
extern int    LongEma                   = 55;
extern int    ShortEma                  = 34;
extern int    SignalEma                 = 13;
extern bool   UseRangeCalculation       = true;
extern bool   divergenceVisible         = false;
extern bool   divergenceOnValuesVisible = true;
extern bool   divergenceOnChartVisible  = true;
extern color  divergenceBullishColor    = Aqua;
extern color  divergenceBearishColor    = Magenta;
extern string divergenceUniqueID        = "kvo diverge1";
extern bool   HistogramOnSlope          = true;
extern bool   Interpolate               = true;

extern bool   alertsOn                  = true;
extern bool   alertsOnSlope             = true;
extern bool   alertsOnCurrent           = false;
extern bool   alertsMessage             = true;
extern bool   alertsSound               = true;
extern bool   alertsEmail               = false;

//
//
//
//
//

double kvo[];
double signal[];
double kvoUpa[];
double kvoUpb[];
double kvoDna[];
double kvoDnb[];
double trend[];
double slope[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;
string shortName;

//+------------------------------------------------------------------
//| 
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,kvoDna); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,kvoDnb); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,kvoUpa); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,kvoUpb); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,kvo);
   SetIndexBuffer(5,signal);
   SetIndexBuffer(6,trend);
   SetIndexBuffer(7,slope);
    
    //
    //
    //
    //
    //
   
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame=="calculateValue");
      
      if (calculateValue)
      {
         int s = StringFind(divergenceUniqueID,":",0);
               shortName = divergenceUniqueID;
               divergenceUniqueID = StringSubstr(divergenceUniqueID,0,s);
               return(0);
      }            
      timeFrame = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
      
      
      
      shortName = divergenceUniqueID+": "+timeFrameToString(timeFrame)+"  Klinger volume oscillator ("+LongEma+","+ShortEma+","+SignalEma+")";
      IndicatorShortName(shortName);
return(0);
}
   
//
//
//
//
//
   
int deinit()
{
   
   int lookForLength = StringLen(divergenceUniqueID);
   
   for (int i=ObjectsTotal()-1; i>=0; i--) {
   
   string objectName = ObjectName(i);
   if (StringSubstr(objectName,0,lookForLength) == divergenceUniqueID) ObjectDelete(objectName);
   }
   
return(0);
}

//+------------------------------------------------------------------
//| 
//+------------------------------------------------------------------
//
//
//
//
//

double work[][6];
#define _price 0
#define _trend 1
#define _dm    2
#define _cm    3
#define _ema1  4
#define _ema2  5

//
//
//
//
//

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);
         if (returnBars)  { kvoDna[0] = limit+1; return(0); }
   
   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {     

      double alpha1 = 2.0/(1.0+LongEma);
      double alpha2 = 2.0/(1.0+ShortEma);
      double alpha3 = 2.0/(1.0+SignalEma);
   
      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         work[r][_price] = (High[i]+Low[i]+Close[i])/3.0;
         work[r][_trend] = work[r-1][_trend];
         
            if (work[r][_price]>work[r-1][_price]) work[r][_trend] =  1;
            if (work[r][_price]<work[r-1][_price]) work[r][_trend] = -1;
         
            //
            //
            //
            //
            //
            
            double volume;
               if (UseRangeCalculation)
               {
                  work[r][_dm] = High[i]-Low[i];
                     if (work[r][_trend]==work[r-1][_trend])
                           work[r][_cm] = work[r-1][_cm]+work[r][_dm];
                     else  work[r][_cm] = work[r-1][_dm]+work[r][_dm];
                  volume = work[r][_trend]*Volume[i] * MathAbs(2.0*work[r][_dm]/work[r][_cm] - 1.0);
               }
               else volume = work[r][_trend]*Volume[i];

            //
            //
            //
            //
            //
            
            work[r][_ema1] = work[r-1][_ema1]+alpha1*(volume-work[r-1][_ema1]);
            work[r][_ema2] = work[r-1][_ema2]+alpha2*(volume-work[r-1][_ema2]);
            kvo[i]         = work[r][_ema2]-work[r][_ema1];
            if (SignalEma>1) signal[i] = signal[i+1]+alpha3*(kvo[i]-signal[i+1]);
            kvoDna[i] = EMPTY_VALUE;
            kvoDnb[i] = EMPTY_VALUE;
            kvoUpa[i] = EMPTY_VALUE;
            kvoUpb[i] = EMPTY_VALUE; 
            trend[i]  = trend[i+1];
            slope[i]  = slope[i+1];
            
            if (kvo[i] > 0)        trend[i] =  1;
            if (kvo[i] < 0)        trend[i] = -1;
            if (kvo[i] > kvo[i+1]) slope[i] =  1;
            if (kvo[i] < kvo[i+1]) slope[i] = -1;
            
            //
            //
            //
            //
            //
            
               
            if (divergenceVisible)
               {
               CatchBullishDivergence(kvo,i);
               CatchBearishDivergence(kvo,i);
               }
                                     
               if (HistogramOnSlope)
               {
                  if (trend[i]== 1 && slope[i] == 1) kvoUpa[i] = kvo[i];
                  if (trend[i]== 1 && slope[i] ==-1) kvoUpb[i] = kvo[i];
                  if (trend[i]==-1 && slope[i] ==-1) kvoDna[i] = kvo[i];
                  if (trend[i]==-1 && slope[i] == 1) kvoDnb[i] = kvo[i];
               }
               else
               {                  
                  if (trend[i]== 1) kvoUpa[i] = kvo[i];
                  if (trend[i]==-1) kvoDna[i] = kvo[i];
               }
               
      }
      manageAlerts();
      return(0);
   }      
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
             
         for(i=limit; i>=0; i--)
         {
         
            int y = iBarShift(NULL,timeFrame,Time[i]);
                kvo[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LongEma,ShortEma,SignalEma,UseRangeCalculation,divergenceVisible,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,shortName,Interpolate,4,y);
                signal[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LongEma,ShortEma,SignalEma,UseRangeCalculation,divergenceVisible,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,shortName,Interpolate,5,y);
                trend[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LongEma,ShortEma,SignalEma,UseRangeCalculation,divergenceVisible,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,shortName,Interpolate,6,y);
                slope[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",LongEma,ShortEma,SignalEma,UseRangeCalculation,divergenceVisible,divergenceOnValuesVisible,divergenceOnChartVisible,divergenceBullishColor,divergenceBearishColor,shortName,Interpolate,7,y);
                kvoDna[i] = EMPTY_VALUE;
                kvoDnb[i] = EMPTY_VALUE;
                kvoUpa[i] = EMPTY_VALUE;
                kvoUpb[i] = EMPTY_VALUE; 
                
                //
                //
                //
                //
                //
      
                if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

                //
                //
                //
                //
                //

                datetime time = iTime(NULL,timeFrame,y);
                   for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
                   for(int k = 1; k < n; k++)
                   {
                      kvo[i+k]    = kvo[i]    + (kvo[i+n]    - kvo[i])    * k/n;
                      signal[i+k] = signal[i] + (signal[i+n] - signal[i]) * k/n;
          }
          }
          for (i=limit;i>=0;i--)
          {
             if (HistogramOnSlope)
               {
                  if (trend[i]== 1 && slope[i] == 1) kvoUpa[i] = kvo[i];
                  if (trend[i]== 1 && slope[i] ==-1) kvoUpb[i] = kvo[i];
                  if (trend[i]==-1 && slope[i] ==-1) kvoDna[i] = kvo[i];
                  if (trend[i]==-1 && slope[i] == 1) kvoDnb[i] = kvo[i];
               }
               else
               {                  
                  if (trend[i]== 1) kvoUpa[i] = kvo[i];
                  if (trend[i]==-1) kvoDna[i] = kvo[i];
               }
               
      }
      manageAlerts();
      return(0);
   }      
   
   //+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      
      //
      //
      //
      //
      //
      
      if (alertsOnSlope)
      {
         if (slope[whichBar] != slope[whichBar+1])
         {
            if (slope[whichBar] == 1) doAlert(whichBar,"slope changed to up");
            if (slope[whichBar] ==-1) doAlert(whichBar,"slope changed to down");
         }         
      }
      else
      {
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] == 1) doAlert(whichBar,"crossed zero line up");
            if (trend[whichBar] ==-1) doAlert(whichBar,"crossed zero line down");
         }         
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," KV oscillator ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Kv oscillator "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

void CatchBullishDivergence(double& values[], int i)
{
   i++;
            ObjectDelete(divergenceUniqueID+"l"+DoubleToStr(Time[i],0));
            ObjectDelete(divergenceUniqueID+"l"+"os" + DoubleToStr(Time[i],0));            
   if (!IsIndicatorLow(values,i)) return;  

   //
   //
   //
   //
   //

   int currentLow = i;
   int lastLow    = GetIndicatorLastLow(values,i+1);
      if (values[currentLow] > values[lastLow] && Low[currentLow] < Low[lastLow])
      {
         if(divergenceOnChartVisible)  DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow],divergenceBullishColor,STYLE_SOLID);
         if(divergenceOnValuesVisible) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],values[currentLow],values[lastLow],divergenceBullishColor,STYLE_SOLID);
      }
      if (values[currentLow] < values[lastLow] && Low[currentLow] > Low[lastLow])
      {
         if(divergenceOnChartVisible)  DrawPriceTrendLine("l",Time[currentLow],Time[lastLow],Low[currentLow],Low[lastLow], divergenceBullishColor, STYLE_DOT);
         if(divergenceOnValuesVisible) DrawIndicatorTrendLine("l",Time[currentLow],Time[lastLow],values[currentLow],values[lastLow], divergenceBullishColor, STYLE_DOT);
      }
}

//
//
//
//
//

void CatchBearishDivergence(double& values[], int i)
{
   i++; 
            ObjectDelete(divergenceUniqueID+"h"+DoubleToStr(Time[i],0));
            ObjectDelete(divergenceUniqueID+"h"+"os" + DoubleToStr(Time[i],0));            
   if (IsIndicatorPeak(values,i) == false) return;

   //
   //
   //
   //
   //
      
   int currentPeak = i;
   int lastPeak = GetIndicatorLastPeak(values,i+1);
      if (values[currentPeak] < values[lastPeak] && High[currentPeak]>High[lastPeak])
      {
         if (divergenceOnChartVisible)  DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak],divergenceBearishColor,STYLE_SOLID);
         if (divergenceOnValuesVisible) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],values[currentPeak],values[lastPeak],divergenceBearishColor,STYLE_SOLID);
      }
      if(values[currentPeak] > values[lastPeak] && High[currentPeak] < High[lastPeak])
      {
         if (divergenceOnChartVisible)  DrawPriceTrendLine("h",Time[currentPeak],Time[lastPeak],High[currentPeak],High[lastPeak], divergenceBearishColor, STYLE_DOT);
         if (divergenceOnValuesVisible) DrawIndicatorTrendLine("h",Time[currentPeak],Time[lastPeak],values[currentPeak],values[lastPeak], divergenceBearishColor, STYLE_DOT);
      }
}

//
//
//
//
//

bool IsIndicatorPeak(double& values[], int i) { return(values[i] >= values[i+1] && values[i] > values[i+2] && values[i] > values[i-1]); }
bool IsIndicatorLow( double& values[], int i) { return(values[i] <= values[i+1] && values[i] < values[i+2] && values[i] < values[i-1]); }

int GetIndicatorLastPeak(double& values[], int shift)
{
   for(int i = shift+5; i<Bars; i++)
         if (values[i] >= values[i+1] && values[i] > values[i+2] && values[i] >= values[i-1] && values[i] > values[i-2]) return(i);
   return(-1);
}

//
//
//
//
//

int GetIndicatorLastLow(double& values[], int shift)
{
   for(int i = shift+5; i<Bars; i++)
         if (values[i] <= values[i+1] && values[i] < values[i+2] && values[i] <= values[i-1] && values[i] < values[i-2]) return(i);
   return(-1);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

void DrawPriceTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
   string   label = divergenceUniqueID+first+"os"+DoubleToStr(t1,0);
   if (Interpolate) t2 += Period()*60-1;
    
   ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, 0, t1+Period()*60-1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, false);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//
//
//
//
//

void DrawIndicatorTrendLine(string first,datetime t1, datetime t2, double p1, double p2, color lineColor, double style)
{
   int indicatorWindow = WindowFind(shortName);
   if (indicatorWindow < 0) return;
   if (Interpolate) t2 += Period()*60-1;
   
   string label = divergenceUniqueID+first+DoubleToStr(t1,0);
   ObjectDelete(label);
      ObjectCreate(label, OBJ_TREND, indicatorWindow, t1+Period()*60-1, p1, t2, p2, 0, 0);
         ObjectSet(label, OBJPROP_RAY, false);
         ObjectSet(label, OBJPROP_COLOR, lineColor);
         ObjectSet(label, OBJPROP_STYLE, style);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs) {
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf) {
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str) {
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--) {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}      
      
      
   

